/************************************************************************
**
** @(#)hgcpixel.cpp		11/17/94		Chris Ahlstrom
**
** C++ main program; edit with tab-stops set to 4.
**
**		Helps me understand pixels on the HGC (for purposes of making
** a cursor).  Derived from sample code in the Borland C++ v 3.10
** Library Reference manual.
**
**		Also useful for creating cursors to use in the rbox_btk module.
**
*************************************************************************/

#include <graphics.h>		// initgraph(), imagesize(), and more
#include <stdlib.h>			// malloc() and free()
#include <stdio.h>			// printf()
#include <string.h>			// strnicmp()
#include <conio.h>			// getch()

#include "scrcombo.h"		// ScreenCombo

extern "C"					// avoid name-mangling
{
	#include "math_aux.h"	// simple math like iround()
	#include "textlog.h"	// getScreenModes() [graphics code!]
}


/************************************************************************
** Dimensions of cursor
**
**		These dimensions are for our most-used monitor, the Hercules.
** For use with VGA, these are scaled appropriately, to keep the cursor
** roughly the same size and design.
**
*************************************************************************/

#define ARROW_WIDTH		32	// width in pixels (X)
#define ARROW_HEIGHT	16	// height in pixels (Y)



/************************************************************************
** Local types and local functions
*************************************************************************/

typedef enum
{
	OPT_EGA,
	OPT_VGA,
	OPT_HERC,
	OPT_MONO,
	OPT_DETECT,
	OPT_BAD

} VideoOptionCode;

typedef enum
{
	SHAPE_ARROW,
	SHAPE_SAUCER,
	SHAPE_DIAMOND,
	SHAPE_CROSSHAIR

} CursorShape;

extern VideoOptionCode getVideoOption
(
	const char *str
);
extern enum graphics_errors graphicsUp
(
	ScreenCombo combo
);
extern int makeImage
(
	const char *filename
);
extern void drawCursor
(
	CursorShape shape,
	int x,
	int y
);


/************************************************************************
** Local data
*************************************************************************/

static CursorShape shapeUsed		= SHAPE_ARROW;
static int vgaResolution			= 0;
static int respPixelX				= 0;
static int respPixelY				= 0;
static const char *fileName			= 0;
static ScreenCombo monitorCombo;

static const char *BIG_HELP =
	"HGCPIXEL v. 1.00 Usage:  hgcpixel [-options] cardtype [file]\n"
	"\n"
	"    Runs some simple graphics tests.  There is no default card-type.\n"
	"The following are supported:\n"
	"\n"
	"    ega    Main VGA or EGA monitor\n"
	"    vga    Main VGA or EGA monitor\n"
	"    herc   Main Hercules monitor\n"
	"    mono   Alternate Hercules monitor (requires patched HERC.BGI file;\n"
	"           see the FACE_TV makefile.bcc).  If two monitors (a VGA and\n"
	"           an HGC) are attached, then this command is like 'herc', but\n"
	"           the VGA screen is not blanked.\n"
	"    detect Use whatever is attached to the computer.\n"
	"\n"
	"If 'file' is given, then the image is saved in C-declaration format.\n"
	"\n"
	"The following options can appear (but only right after the program\n"
	"name:\n"
	"\n"
	"    -a     Cheesy arrow [default]\n"
	"    -c     Crosshair\n"
	"    -d     Diamond shape\n"
	"    -s     Cheesy flying saucer\n"
	"\n"
	;

static const char *videoOptions[] =
{
	"ega",
	"vga",
	"herc",
	"mono",
	"detect",
	0
};


/************************************************************************
** Herc_Alt_driver_far()
**
**   This function is *not* callable... it is here only to allow
** linking in the patched HERC graphics driver.  See the makefile
** (makefile.new).  Note the difference between the two declarations,
** too.
**
**		We must use the far versions, to avoid _TEXT exceeding 64K.
**
*************************************************************************/

extern int far _Cdecl Herc_Alt_driver_far[];
int	far _Cdecl registerfarbgifont(void far *__font);


/************************************************************************
** main()
*************************************************************************/

int
main
(
	int argc,
	char *argv[]
)
{
	int errcode = 0;
	VideoOptionCode device;
	ScreenCombo combo;

	if (argc > 1)
	{
		int argindex = 1;

		while ((argv[argindex][0] == '-') || (argv[argindex][0] == '/'))
		{
			int ch = argv[argindex][1];

			switch (ch)
			{
				case 'a':					// arrow

					shapeUsed = SHAPE_ARROW;
					break;

				case 'c':					// crosshair

					shapeUsed = SHAPE_CROSSHAIR;
					break;

				case 'd':					// diamond

					shapeUsed = SHAPE_DIAMOND;
					break;

				case 's':					// saucer

					shapeUsed = SHAPE_SAUCER;
					break;
			}
			argindex++;
		}

		if ((argv[argindex][0] != '-') && (argv[argindex][0] != '/'))
		{
			device = getVideoOption(argv[argindex]);
			switch (device)
			{
			case OPT_EGA:
			case OPT_VGA:

				combo = BGI_USE_MAIN_VGA;
				vgaResolution = 1;
				break;

			case OPT_HERC:

				combo = BGI_USE_HERC;
				break;

			case OPT_MONO:

				combo = BGI_USE_MONO_PATCH;
				break;

			case OPT_DETECT:				// not fully thought through yet!

				combo = BGI_AUTO_DETECT;
				break;

			case OPT_BAD:

				errcode = 2;
				break;
			}
		}
		argindex++;
		if (argc > argindex)
		{
			if ((argv[argindex][0] != '-') && (argv[argindex][0] != '/'))
				fileName = argv[argindex];
		}
	}
	else
	{
		printf(BIG_HELP);
		errcode = 1;
	}

	if (!errcode)
	{
		enum graphics_errors grerror = graphicsUp(combo);
		if (grerror != grOk)
		{
			errcode = 3;
			printf
			(
				"Graphics error %s.\n"
				"Press any key to halt: ",
				grapherrormsg(grerror)
			);
			getch();
		}
	}

	if (!errcode)
	{
		if (makeImage(fileName) != 0)
			errcode = 4;
	}
	return errcode;
}


/************************************************************************
** getVideoOption()
*************************************************************************/

VideoOptionCode
getVideoOption
(
	const char *str
)
{
	const char * * list = videoOptions;
	VideoOptionCode rvalue = OPT_BAD;
	int count = 0;

	while (*list)
	{
		if (stricmp(*list, str) == 0)
		{
			rvalue = (VideoOptionCode) count;
			break;
		}

		list++;
		count++;
	}
	return rvalue;
}


/************************************************************************
** graphicsUp()
**
**	Puts the main adapter in graphics mode, assuming the VGA
** driver is part of the executable module (see the UTIL.DOC provided
** by Borland, I believe).
**
**		We restrict ourselves to the FAR memory model, and have generated
** far versions of the graphics drivers.  See makefile.bcc for more
** notes.
**
*************************************************************************/

static int graphDriver;
static int graphMode;
static int vgaPixelX	= 640;
static int vgaPixelY	= 200;
static int hercPixelX	= 720;
static int hercPixelY	= 348;

enum graphics_errors
graphicsUp
(
	ScreenCombo combo			// which type of screen to use
)
{
	void *driverptr;			// will point to proper far driver array
	void *fontptr;				// will point to a far stroked BGI font array

	int *gd = &graphDriver;			// use DetectGraph (if 0), or driver
	int *gm = &graphMode;			// if not zero, this is graphics mode
	char *dpath = "";				// no location to look for drivers
	enum graphics_errors errcode;	// a return code

	graphDriver	= DETECT;
	graphMode	= 0;
	detectgraph(gd, gm);						// find basic facts
	errcode = (graphics_errors) graphresult();
	if (errcode != grOk)						// no graphics hardware
		return errcode;

	switch (combo)
	{
	case BGI_USE_MAIN_VGA:

		driverptr	= EGAVGA_driver_far;
		fontptr		= triplex_font_far;
		graphDriver	= VGA;
		graphMode	= VGALO;					// 640 x 200
		respPixelX	= vgaPixelX;
		respPixelY	= vgaPixelY;
		break;

	case BGI_USE_MONO_PATCH:

		driverptr	= Herc_Alt_driver_far;
		fontptr		= triplex_font_far;
		graphDriver	= HERCMONO;
		graphMode	= HERCMONOHI;				// 720 x 348
		respPixelX	= hercPixelX;
		respPixelY	= hercPixelY;
		break;

	case BGI_USE_HERC:

		driverptr	= Herc_driver_far;
		fontptr		= triplex_font_far;
		graphDriver	= HERCMONO;
		graphMode	= HERCMONOHI;
		respPixelX	= hercPixelX;
		respPixelY	= hercPixelY;
		break;

	case BGI_AUTO_DETECT:

		driverptr	= 0;
		fontptr		= 0;
		graphDriver	= DETECT;
		graphMode	= 0;
		Screen *sc = new Screen;					// allocate a struct
		if (sc)
		{
			(void) getScreenModes(sc);				// obtain many parameters
			//respRows		= sc->screen_rows;		// number of text rows
			//respColumns	= sc->screen_columns;	// number of text columns
			respPixelX		= sc->screen_gmax_x;	// pixel depth
			respPixelY		= sc->screen_gmax_y;	// pixel width
			delete sc;								// get rid of struct
		}
		break;
	}

	if (registerfarbgidriver(driverptr) < 0)
		errcode = grInvalidDriver;
	else if (registerfarbgifont(fontptr) < 0)
		errcode = grInvalidFont;

	if (!errcode)
	{
		initgraph(gd, gm, dpath);					// initialize graphics board
		errcode = (graphics_errors) graphresult();	// check the status
		monitorCombo = combo;
	}
	return errcode;
}


/************************************************************************
** makeImage()
*************************************************************************/

static int arrowXWidth;
static int arrowXHalf;
static int arrowYWidth;
static int arrowYHalf;

const char *HEADER_EXPLANATION =
	"\t/* two words (width-1 and height-1)\t*/\n"
	;

int
makeImage
(
	const char *filename
)
{
	int errcode = 0;
	int x;
	int y;
	int maxx;
	int maxy;
	unsigned size;
	void *arrow;
	int left;
	int right;
	int top;
	int bottom;
	int showcount = 0;

	maxx = getmaxx();
	maxy = getmaxy();
	x = 0;
	y = maxy / 2;

	arrowXWidth	= ARROW_WIDTH;				// e.g. 32
	arrowYWidth	= ARROW_HEIGHT;				// e.g. 16

	if (vgaResolution)
	{
		double xw = (double) arrowXWidth * vgaPixelX / (double) hercPixelX;
		double yw = (double) arrowYWidth * vgaPixelY / (double) hercPixelY;

		arrowXWidth = iround(xw);
		arrowYWidth = iround(yw);
	}

	arrowXHalf	= arrowXWidth / 2;			// e.g. 16
	arrowYHalf	= arrowYWidth / 2;			// e.g.  8

	left	= x;
	right	= x + arrowXWidth - 1;
	top		= y - arrowYHalf;
	bottom	= y + arrowYHalf - 1;

	drawCursor(shapeUsed, x, y);

	size = imagesize						// calculate size of the image
	(
		left, top, right, bottom
	);

	arrow = malloc(size);					// memory to hold the image

	if (arrow)
	{
		getimage							// grab the image
		(
			left, top, right, bottom, arrow
		);

		if (filename)
		{
			FILE *handle = fopen(filename, "wt");

			if (handle)
			{
				unsigned char *source = (unsigned char *) arrow;
				unsigned count = 0;
				unsigned linesize = arrowXWidth / 8;

				if (vgaResolution)
					linesize *= 2;

				if (linesize < 4)
					linesize = 4;
				else if (((linesize < 8) && (linesize != 4)) || (linesize > 8))
					linesize = 8;

				(void) fprintf
				(
					handle,
					"Our calculated size of image is "
					"%d width x %d height = %d pixels (%d bytes).\n\n",
					arrowXWidth, arrowYWidth, (arrowXWidth * arrowYWidth),
					((arrowXWidth * arrowYWidth) / 8)
				);
				(void) fprintf
				(
					handle,
					"const unsigned char imageBitMap[%d] =\n"
					"{\n\t",
					size
				);

				/********************************************************
				** The first 4 bytes show the index values for the
				** width and height of the image (in pixels, minus one).
				*********************************************************/

				for (int hi = 0; hi < 4; hi++)
				{
					(void) fprintf(handle, "0x%02X, ", (unsigned int) *source);
					source++;
					count++;
				}
				(void) fprintf(handle, HEADER_EXPLANATION);

				while (count < size)
				{
					if (((count-4) % linesize) == 0)
						(void) fprintf(handle, "\n\t");

					(void) fprintf(handle, "0x%02X, ", (unsigned int) *source);
					source++;
					count++;
				}

				(void) fprintf(handle, "\n};\n\n");
				(void) fclose(handle);
			}
			else
				errcode = 1;
		}

		while (!kbhit())
		{
			if (showcount > 0)				// keep first image around
			{
				putimage					// erase old image
				(
					x,						// left
					y - arrowYHalf,			// top,
					arrow,					// bitmap,
					XOR_PUT					// bit opcode
				);
			}
			else
				showcount++;

			x += arrowXWidth + 8;
			if (x >= maxx)
			{
				x = arrowXWidth + 8;
				y += arrowYWidth + 4;
				if (y >= maxy)
					y = arrowYHalf + 4;
			}
			putimage						// plot new image
			(
				x,							// left
				y - arrowYHalf,				// top,
				arrow,						// bitmap,
				XOR_PUT						// bit opcode
			);
			for (int dawdle = 0; dawdle < 100; dawdle++)
				if (kbhit())
					break;
		}
		free(arrow);	
	}
	else
		errcode = 2;

	closegraph();

	if (errcode == 1)
		printf("Failure opening file '%s'\n", filename);
	else if (errcode == 2)
		printf("Allocation failure!!!\n");

	return errcode;
}



/************************************************************************
** drawCursor()
**
**				   x+arrowHalf
**				   y-arrowHalf
**				   |\
**				   | \
**				   |  \
**				   |   \
**				   |    \
**				   |     \
**				   |      \
**				   |       \
**				   |        \
**				   |         \
**	    x----------o----------x+arrowXWidth-1
**				   |         /
**				   |        /
**				   |       /
**				   |      /
**				   |     /
**				   |    /
**				   |   /
**				   |  /
**				   | /
**				   |/
**				   x+arrowXHalf
**				   y+arrowYHalf-1
**
**
**		The drawCursor() function  also draw two more bars, to make a
** saucer-like diamond shape, or it can draw a filled diamond.
**
*************************************************************************/

void
drawCursor
(
	CursorShape shape,
	int x,
	int y
)
{
	int xleft	= 0;
	int xright	= arrowXWidth - 1;		// covers the full width of arrow
	int xmid	= arrowXHalf;

	int ymid	= 0;
	int ytop	= arrowYHalf;
	int yfull	= arrowYWidth - 1;		// covers the full height of arrow
	int ybottom	= arrowYHalf - 1;

	if ((shape == SHAPE_ARROW) || (shape == SHAPE_SAUCER))
	{
		moveto(x, y);							// get to left corner

		linerel(	xright,	ymid		);		// horizontal line
		linerel(	-xmid,	-ytop		);		// up and to left
		linerel(	xleft,	yfull		);		// straight down to bottom
		linerel(	xmid,	-ybottom	);		// up and to right

		if (shape == SHAPE_SAUCER)				// add two more lines
		{
			moveto(x, y);
			linerel(	xmid,	-ytop	);		// up and to right

			moveto(x, y);
			linerel(	xmid,	ybottom	);		// down and to right
		}
	}
	else if (shape == SHAPE_DIAMOND)
	{
		const int corners = 4;
		int polygon[2*corners];

		polygon[0]	= x;			polygon[1] = y;
		polygon[2]	= x + xmid;		polygon[3] = y - ytop;
		polygon[4]	= x + xright;	polygon[5] = y;
		polygon[6]	= x + xmid;		polygon[7] = y + ybottom;

		setfillstyle(SOLID_FILL, getmaxcolor());
		fillpoly(corners, polygon);
	}
	else if (shape == SHAPE_CROSSHAIR)
	{
		moveto(x, y);
		linerel(	xright,	ymid		);		// horizontal line
		moverel(	-xmid,	-ytop		);		// up and to left
		linerel(	xleft,	yfull		);		// straight down to bottom
		circle(x+xmid-1, y, arrowXWidth/8);		// little circle
	}
}

