/************************************************************************
**
** @(#)rbox_afc.h		11/11/94		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
**	C++; edit using tab-stops = 4.
**
**		Declares structures for holding information about lights in
** an mAFC response-box implemented on-screen.  This module defines
** just one basic kind of mAFC display, though many wrinkles can
** be supported with this file.
**
**		Also, the module supports either use of Turbo Vision windows
** or use of Borland BGI graphics.  For using TV, #define the symbol
** "TV_PALETTES", otherwise don't #define it.
**
**		Please see rbox2afc.hdt for an example of a static data
** structure that employs our scheme.
**
*************************************************************************/

#if !defined(RBOX_AFC_h)							// { RBOX_AFC_h
#define RBOX_AFC_h


/************************************************************************
** Old #defines
**
**		I'm sure these will go out of commission soon, even if they
** remain here as a subtle reminder of the methods.
**
*************************************************************************/

#define BRESP_LIGHTS_MAX	16		// number of bits in one short integer
#define BRESP_INTV_BITS		 4		// shift'em 4 bits left for use
#define BRESP_OFF			 0		// the light is off
#define BRESP_ON			 1		// the light is on


/************************************************************************
** ResponseBoxSet
**
**		Constants for the implementation of light panels.  Used by
** startScreen() routines to determine whether to go on or quit.
**
*************************************************************************/

typedef enum
{
	BOX_RESPONSE_NORMAL = 0,
	BOX_RESPONSE_QUIT

} ResponseBoxSet;


/************************************************************************
** ResponseBoxLightIndex
**
**		All ResponseBoxInterior-derived classes must support this
** minimum configuration of lights.  Basically, even if the derived
** class does some other screen drawing, at minimum it will show the
** following sequence:
**
**		Set up the background color (a large colored square)
**
**		Set up the foreground frame (either the same as the back-ground,
**		or perhaps a specially-drawn grid of lines)
**
**		Turn on a ready light (coincident with the start of a trial)
**
**		Turn the an interval light for each interval of presentation
**		(anywhere from 1 to 3 intervals)
**
**		Turn on an answer light (to tell the subject to respond)
**
**		Provide feedback, or a right/wrong indicator.
**
** Each derived object must be prepared to raise an indicator of an
** internal problem, too (the failure light).
**
**		In your coding, be sure to note that the color used for the
** BOX_BACKGROUND_LIGHT is used for two purposes:
**
**		1.	Drawing the whole background.
**
**		2.	Drawing the off-version of each light.
**
*************************************************************************/

typedef enum
{
	BOX_FIRST_LIGHT			= 0,				// useful in for() loops
	BOX_BACKGROUND_LIGHT	= BOX_FIRST_LIGHT,	// optional background painter
	BOX_FRAME_LIGHT,							// optional foreground dressing
	BOX_READY_LIGHT,							// "prepare for stimulus!"
	BOX_ANSWER_LIGHT,							// "you can now answer..."
	BOX_RESPONSE_1,								// "you chose this interval"
	BOX_WRONG_RESPONSE		= BOX_RESPONSE_1,	// "you were wrong!"
	BOX_RESPONSE_2,								// "you chose this interval"
	BOX_RIGHT_RESPONSE		= BOX_RESPONSE_2,	// "you were right!"
	BOX_RESPONSE_3,								// "you chose this interval"
	BOX_INTERVAL_1,								// "now presenting interval 1"
	BOX_INTERVAL_2,								// "now presenting interval 2"
	BOX_INTERVAL_3,								// "now presenting interval 3"
	BOX_BANNER_LIGHT,							// "Response Box" banner
	BOX_FAILURE_LIGHT,							// "soft/hardware failure"
	BOX_INSTR_1_LIGHT,							// "instructions 1"
	BOX_INSTR_2_LIGHT,							// "instructions 2"

	BOX_LIGHT_MAX								// useful in for() loops

} ResponseBoxLightIndex;


/************************************************************************
** ResponseBoxLight
**
**		Actually just a quick way to specify how and where to draw
** a simple geometric shape (usually a rectangle).
**
**		Note that "pixel" below actually refers to our virtual
** coordinate system (0 to 10000).
**
*************************************************************************/

typedef struct
{
	int fillstyle;				// style of graphics background
	unsigned color;				// color of the response light
	int x0, y0;					// pixel coordinate of upper left
	int x1, y1;					// pixel coordinate of lower right
	int status;					// is light on or off?

} ResponseBoxLight;


/************************************************************************
** ResponseBoxText
**
**		Actually just a quick way to specify how and where to draw
** a string of text.
**
*************************************************************************/

typedef struct
{
	int font;					// font to use for the text
	int direction;				// direction the lettering should go
	int charsize;				// relative size of the character
	int x0, y0;					// location of the start of the text
	const char *textstring;		// string to draw (not write)
	int status;					// is light on or off?

} ResponseBoxText;


/************************************************************************
** Parameters of mAFC response lights
**
**		We define the parameters of the response-box lights.  The
** representation is very simple and independent of the device used
** to draw the lights.  We define a virtual screen of 10000 in width,
** and a 10000 in height.  Then we locate the coordinates in
** terms of these maximal dimensions.  Basically, the numbers
** below respresent to coordinates in terms of percentage of
** full-screen, times 100.
**
**		Note that the virtualBox will be our standard, and we
** will use that for converting all other representations into
** the units proper for the display device.
**
**	--------------------
**	Guide to conversions
**	--------------------
**
**		The "screen_" values are as per screen.c and screen.h in the
** FACE library.
**
**	1.  Converting virtual coordinates to characters:
**
**		a.  screen_x = (virtual_x / VIRTmax) * screen_columns
**		b.  screen_y = (virtual_y / VIRTmax) * screen_rows
**
**	2.  Converting virtual coordinates to pixels:
**
**		a.  screen_x = (virtual_x / VIRTmax) * screen_gmax_x
**		b.  screen_y = (virtual_y / VIRTmax) * screen_gmax_y
**
**	3.	Converting VGA pixel dimensions into EGA pixel dimensions:
**
**		a. X-values are such that EGA_X = VGA_X
**		b. Y-values are such that EGA_X = (3/4)VGA_X
**
*************************************************************************/

#define VIRTmax	10000		// our standard (100 is too coarse)


/************************************************************************
** Turbo Vision color support
**
**		Turbo Vision handles colors as object characteristics, rather
** than as attribute bytes.   You pick "the same color as normal window
** color", rather than the attribute "yellow on blue", for example.
** The main application defines a large palette that everything
** in Turbo Vision will use.  Each object defines a short array that
** contains indices into this large palette.
**
**		The programmer uses getColor to select an index into the
** local object's palette, and the local object's palette points
** to the large palette.
**
**		We set things up so that we can select either attribute bytes
** or TV palette indices at compile-time.
**
**		Further explanation of Turbo Vision palettes:
**
**		TApplication inherits cpColor (see APP.H) from TProgram, and
** I've decided (for now) to limit the choice of colors to this palette.
** See p. 342-3 of the TV manual for a list of the colors.  At present,
** we use only colors from one section (top of p. 343).  Also, we
** use the foreground colors (using character RCELL, which fills the
** whole foreground of the cell).  We could use RBACK (which doesn't
** fill the foreground with the background color.
**
**		None of the colors in cpColor has the bright red we desire
** for the READY_LIGHT.  Instead, yellow is used, which makes it
** at least a warning in American culture.
**
*************************************************************************/

#if defined(TV_PALETTES)							// { TV_PALETTES

#include "tv_pale.h"

#else												// }{ TV_PALETTES

#include <graphics.h>					// TV graphics functions

#define RESPONSE_COLOR			WHITE	// or BROWN	or CYAN
#define INTERVAL_COLOR			WHITE
#define READY_COLOR				LIGHTRED
#define ANSWER_COLOR			LIGHTGREEN
#define BACKGROUND_COLOR		BROWN

#define OUR_EMPTY_FILL			EMPTY_FILL
#define OUR_SOLID_FILL			SOLID_FILL

#define MONO_VISIBLE_COLOR		WHITE	// any color except black will do
#define MONO_RESPONSE_COLOR		MONO_VISIBLE_COLOR	// or BLACK
#define MONO_INTERVAL_COLOR		MONO_VISIBLE_COLOR
#define MONO_READY_COLOR		MONO_VISIBLE_COLOR
#define MONO_ANSWER_COLOR		MONO_VISIBLE_COLOR
#define MONO_BACKGROUND_COLOR	BLACK

#endif 												// } TV_PALETTES


#define AXIS_BOUNDS		10000			// same as VIRTmax

#define LWDTH	2660					// width of response/interval
#define LHGTH	2000					// height of response/interval
#define FWDTH	 780					// width of protocol lights
#define FHGTH	 780					// height of protocol lights

#define LEFTX0	1560					// basic left X location
#define LEFTX1	(LEFTX0 + LWDTH)
#define RGHTX0	5780					// basic right X location
#define RGHTX1	(RGHTX0 + LWDTH)

#define INTVY0	3340					// basic Y location
#define INTVY1	(INTVY0 + LHGTH)
#define RESPY0	(INTVY1 + LHGTH)		// offset from interval light
#define RESPY1	(RESPY0 + LHGTH)

#define READX0	 780					// basic X for protocol lights
#define READX1	(READX0 + FWDTH)
#define ANSWX0	8440
#define ANSWX1	(ANSWX0 + FWDTH)

#define PROTY0	 780					// basic Y for protocol lights
#define PROTY1	(PROTY0 + FHGTH)


/************************************************************************
** Static data
**
**		One of the modules must #define USE_RESPLIGHT_DATA before
** including rbox_afc.h, in order that the following externals be
** defined.  The FACE_TV module that includes it is rbox_bgi.cpp.
**
*************************************************************************/

extern ResponseBoxLight resplightParameters[];
extern int resplightCount;
extern ResponseBoxText resptextParameters[];
extern int resptextCount;
extern unsigned afcBitMaskList[];
extern int allLightCount;

#if defined(USE_RESPLIGHT_DATA)		// allow inclusion of static data

#include "rbox2afc.hdt"				// resplightParameters[], etc.

#endif


#endif												// } RBOX_AFC_h

