/************************************************************************
**
** @(#)rbox_bas.cpp		01/07/96		Chris Ahlstrom
**
** C++; edit with tab-stops at 4.
**
**		Provides the interface for ResponseBoxDevice (rbox_dbg modules)
** and ResponseBGIDevice (rbox_bgi modules).
**
**		All ResponseBoxDeviceBase descendants will have the same
** type of button devices available to them, but different types of
** screen representations of response lights and text.
**
*************************************************************************/

#define RBOX_BAS_cpp

#include <string.h>			// strncpy()

#include "mkbutton.h"		// ResponseButton creation routine
#include "rbox_bas.h"		// ResponseBoxDeviceBase class


/************************************************************************
** ResponseBoxDeviceBase
*************************************************************************/

ResponseBoxDeviceBase::ResponseBoxDeviceBase
(
	ResponseType device,		// e.g. MOUSE_RESPONSE, KEYBOARD_RESPONSE
	const Rectangle &bounds,	// size occupied on display
	const char *atitle,			// later, could write at top of screen
	ScreenCombo combo,			// mouse/monitor combination
	ResponseFeature style,		// response-box style
	int itemcount				// number of intervals (1 to 3)
) :
	boxBounds		(bounds),						// size occupied on display
	buttons			(makeResponseButton(device)),	// class-universal
	screenCombo		(combo),						// for storage
	responseStyle	(style),						// response-box style
	itemCount		(itemcount),					// no. of important items
	subInterior		(0)								// class-peculiar
{
	boxTitle[0] = 0;
	if (atitle != 0)
	{
		(void) strncpy(boxTitle, atitle, 32);
		boxTitle[32-1] = 0;
	}
}


/************************************************************************
** ~ResponseBoxDeviceBase
*************************************************************************/

ResponseBoxDeviceBase::~ResponseBoxDeviceBase()
{
	if (subInterior)
		delete subInterior;
	if (buttons)
		delete buttons;
}


/************************************************************************
** startScreen
**
**		Turns on the main display, and, if it must, waits for a response
** from the buttons device.
**
**		In spite of the improbability of changing this routine, it
** is virtual.  [It did get changed, ca 12/30/95, in rbox_nul.cpp/h!!!]
**
*************************************************************************/

ResponseBoxSet
ResponseBoxDeviceBase::startScreen
(
	int proceed
)
{
	ResponseBoxSet status = BOX_RESPONSE_NORMAL;

	if (!subInterior || !buttons)
		status = BOX_RESPONSE_QUIT;
	else
	{
		(void) subInterior->startState();			// first display
		if (!proceed)								// do we need to wait?
		{
			if (buttons->waitKey() == BOTH_BUTTONS)	// wait for subject
				status = BOX_RESPONSE_QUIT;
		}
		(void) subInterior->responseOff();			// turn off all lights
	}
	return status;
}


/************************************************************************
** Encapsulating routines
**
**		Although many of these routines will not change in derived
** classes, some will, so they all are made virtual.
**
*************************************************************************/

void
ResponseBoxDeviceBase::readyLight ()
{
	if (subInterior)
		(void) subInterior->readyLight();
}


void
ResponseBoxDeviceBase::responseOff ()
{
	if (subInterior)
		(void) subInterior->responseOff();
}


void
ResponseBoxDeviceBase::intervalLight
(
	unsigned interval
)
{
	if (subInterior)
	{
		if (interval > itemCount)
			interval = 1;

		(void) subInterior->intervalLight((int) interval);
	}
}


void
ResponseBoxDeviceBase::answerLight ()
{
	if (subInterior)
		(void) subInterior->answerLight();
}


int
ResponseBoxDeviceBase::responseFeedback
(
	int stimulus
)
{
	int tkey;

	if (!subInterior && !buttons)
		tkey = BOTH_BUTTONS;
	else
	{
		(void) subInterior->answerLight();		// light answer light
		if (buttons)
			tkey = buttons->waitKey();			// wait for subject
		(void) subInterior->responseOff();		// turn off answer light

		if
		(
			(tkey != BOTH_BUTTONS)				// exit button(s)?
				&&
			(itemCount > 1)					// yes/no response?
		)
		{
			if (tkey == BUTTON_3)				// 3-button mouse or keyboard
				tkey = 3;						// re-map it

			(void) subInterior->feedbackLight	// light feedback light
			(
				stimulus, tkey					// each is 1 to 3
			);
		}
	}
	return tkey;
}


void
ResponseBoxDeviceBase::failureLight ()
{
	if (subInterior)
		(void) subInterior->failureLight();
}


void
ResponseBoxDeviceBase::pause ()
{
	if (buttons)
		(void) buttons->waitKey();				// wait for subject
}


int
ResponseBoxDeviceBase::isKeyed ()
{
	return (buttons->readKey() != 0);
}


int
ResponseBoxDeviceBase::keyAbort ()
{
	return (buttons->getLastResponse() == BOTH_BUTTONS);
}
