/************************************************************************
**
** @(#)rbox_btk.h		12/08/94		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
**	C++; edit with tab-stops = 4.
**
**		Defines the ResponseBGITokenInterior class.  We do not want
** to derive it from the ResponseBoxInterior class, because that class
** sets up a complex scheme using bitmasks and lights.  Our speech
** token screen will be somewhat less flexible [it is based on
** old code of John Wygonski's] -- more of it is hardwired -- and so
** we need to implement all the public light-control functions in
** a completely different way.  Here is a quick map of the current
** hierarchy:
**
**						ResponseInteriorBase [rbox_inb]
**							^			^
**							|			|
**							|			|
**		ResponseBGITokenInterior	ResponseBoxInterior
**			^							^			^
**			"							|			|
**			"							|			|
**			"			ResponseBGIInterior		ResponseTVInterior
**			"				^						^
**			"				"						"
**			"				"						"		TView
**		BGIManager=========="						"		  ^
**													"		  |
**													"		  |
**										ResponseBoxTView------'
**
** Note in the above diagram that the BGI classes include a BGIManager
** in a "has-a" relationship.  Note that this hierarchy diagrams only
** the interiors of the response boxes.
**
** Here are the names of the modules, for reference:
**
**	ResponseInteriorBase		rbox_inb	ABC class
**	ResponseBoxInterior			rbox_int	Simple lights with bitmasks
**	ResponseBGIInterior			rbox_bgi	Bitmasked lights for BGI
**	ResponseBGITokenInterior	rbox_btk	Hardwired grid of phonemes
**	ResponseTVInterior			rbox_tv?	Bitmasked lights for TV???
**
*************************************************************************/

#if !defined(RBOX_BTK_h)								// { RBOX_BTK_h
#define RBOX_BTK_h

#include "bgibase.h"		// BGIManager class
#include "rbox_inb.h"		// ResponseBoxInteriorBase class
#include "toklayo.h"		// TokenLayout structure


/************************************************************************
** ResponseBGITokenInterior
**
**		Since a ResponseBox is allocated in the constructor, we really
** need to add an assignment operator and copy constructor. Later...
** (famous last word).  Or make them private.
**
**		This ResponseBoxInterior has to do much real work.
**
*************************************************************************/

class ResponseBGITokenInterior : public ResponseBoxInteriorBase
{

public:

	ResponseBGITokenInterior				// constructor
	(
		TokenLayout *tokens,				// box-numbers and strings
		int tokencount,						// number of them
		const char *atitle,					// optional title
		ScreenCombo combo
	);

	virtual ~ResponseBGITokenInterior ();	// destructor

	virtual int drawThis ();				// redraws whole interior
	virtual int startState ();				// turn on all the lights
	virtual int intervalLight
	(
		int interval,
		int onstate = TURN_THEM_OFF
	);
	virtual int responseLight
	(
		int response,
		int onstate = TURN_THEM_OFF
	);

	virtual int responseOff					// turn off all but a couple
	(
		int allofthem = 0					// turn those off, too
	);
	virtual int readyLight
	(
		int onstate = TURN_THEM_OFF
	);
	virtual int failureLight
	(
		int onstate = TURN_THEM_OFF
	);
	virtual int bannerLight
	(
		int onstate = TURN_THEM_OFF
	);
	virtual int answerLight
	(
		int onstate = TURN_THEM_OFF
	);
	virtual int wrongLight
	(
		int onstate = TURN_THEM_OFF
	);
	virtual int rightLight
	(
		int onstate = TURN_THEM_OFF
	);
	virtual int feedbackLight
	(
		int interval,
		int response
	);
	virtual void cursorOn					// activate our own cursor
	(
		double xfraction = -1.0,			// 0 to 1, relative start of cursor
		double yfraction = -1.0				// 0 to 1, relative start of cursor
	);
	virtual void cursorOff ();				// deactivate our own cursor
	virtual void cursorMove					// move cursor to new place
	(
		int x,
		int y
	);
	virtual int getBoxNumber
	(
		int x,
		int y
	);

protected:									// internal support

	virtual void allOff ();					// utility function

	virtual int drawSpecial					// redraw special light
	(
		int lightnumber,
		unsigned onstate
	);
	virtual int drawText
	(
		unsigned lpboffset,
		unsigned onstate
	);
	virtual int drawGraphic
	(
		unsigned lpboffset,
		unsigned onstate
	);

protected:

	void drawBoxCorrect ();
	void drawBoxIncorrect ();
	void drawBoxClick ();
	void drawGrid
	(
		int sx,
		int ex,
		int sy,
		int ey
	);
	void fillTokenBox
	(
		char *ostr,
		int box
	);
	void clearTokenBox
	(
		char *ostr,
		int box
	);

protected:

	TokenLayout *Tokens;				// box-numbers and strings
	int tokenCount;						// the number of them
	int monoChrome;						// set when using Hercules
	int lineColor;						// color of grid lines
	int intervalColor;					// general purpose color
	int readyColor;						// color of ready light
	int answerColor;					// color of answer light
	int backgroundColor;				// color of whole screen
	BGIManager *bgiControl;				// for video control
	int xAxisSpacing;					// spacing of boxes on the x axis
	int yAxisSpacing;					// spacing of boxes on the y axis
	int xLineOffset;					// offset of lines from left
	int yLineOffset;					// offset of lines from top
	int xPhonemeOffset;					// offset of phoneme from left of box
	int yPhonemeOffset;					// offset of phoneme from top of box
	int handleOwnCursor;				// set to avoid using MOUSE's cursor
	int monoCoordinates;				// true if mouse uses mono coordinates
	const unsigned char *cursorImage;	// the current cursor image
	int bitWidth;						// width of current cursor (pixels)
	int bitHeight;						// height of current cursor (pixels)
	int minMouseX;						// area mouse is restricted to
	int minMouseY;
	int maxMouseX;
	int maxMouseY;
	int oldMouseX;						// previous location of mouse
	int oldMouseY;
	int imageOn;						// true if image can be erased

};


#endif													// } RBOX_BTK_h
