/************************************************************************
**
** @(#)rbox_dtk.cpp		12/08/94		Chris Ahlstrom
**
** C++; edit with tab-stops at 4.
**
**		This module defines the base class ResponseBGITokenDevice.
**
**		The class works by creating a response-window object for
** presenting visual feedback, and a button object for accepting
** responses from the user.
**
*************************************************************************/

#define RBOX_DTK_cpp

#include <graphics.h>		// putimage()

#include "mkbutton.h"		// ResponseButton creation class
#include "rbox_dtk.h"		// ResponseBGITokenDevice class


/************************************************************************
** ResponseBGITokenDevice
**
**		I'm not sure how to subdivide the work.
**
*************************************************************************/

ResponseBGITokenDevice::ResponseBGITokenDevice
(
	TokenLayout *tokens,		// list of token-strings & their box-numbers
	int tokencount,				// size of the list in items
	ResponseType device,		// type of device for user's responses
	const Rectangle &bounds,	// (currently not used)
	const char *atitle,			// later, could write at top of screen
	ScreenCombo combo,			// mouse/monitor combination
	int selectionfeedback,		// if set, blink the true token
	int tactilefeedback			// if set, blink the subject's selection
) :
	ResponseBoxDeviceBase
	(
		device, bounds, atitle, combo, RF_STANDARD, tokencount
	),
	tokenList			(tokens),
	tokenCount			(tokencount),
	selectionFeedback	(selectionfeedback),
	tactileFeedback		(tactilefeedback),
	oldX				(9999),
	oldY				(9999)
{
	subInterior = new ResponseBGITokenInterior
	(
		tokens, tokencount, atitle, combo
	);
	if (subInterior)
		subInterior->cursorOn(0.5, 0.5);	// set up the cursor in middle
}


/************************************************************************
** ~ResponseBGITokenDevice
*************************************************************************/

ResponseBGITokenDevice::~ResponseBGITokenDevice()
{
}


/************************************************************************
** startScreen()
**
**		This version of startScreen() must not turn off the lights
** after the subject acknowledges readiness.
**
*************************************************************************/

ResponseBoxSet
ResponseBGITokenDevice::startScreen
(
	int proceed
)
{
	ResponseBoxSet status = BOX_RESPONSE_NORMAL;

	if (!subInterior || !buttons)
		status = BOX_RESPONSE_QUIT;
	else
	{
		(void) subInterior->startState();			// first display
		if (!proceed)								// do we need to wait?
		{
			if (buttons->waitKey() == BOTH_BUTTONS)	// wait for subject
				status = BOX_RESPONSE_QUIT;

			subInterior->readyLight(LIGHT_OFF);
			subInterior->answerLight(LIGHT_OFF);
			subInterior->bannerLight(LIGHT_OFF);
		}
	}
	return status;
}


/************************************************************************
** intervalLight()
**
**		We have to override this function so it can call interval-light
** with an onstate value of LIGHT_ON, to cause the light to flash.
**
*************************************************************************/

void
ResponseBGITokenDevice::intervalLight
(
	unsigned interval
)
{
	if (subInterior)
	{
		if (interval > itemCount)
			interval = 1;

		(void) subInterior->intervalLight((int) interval, LIGHT_ON);
	}
}


/************************************************************************
** responseFeedback()
**
**		This version must not turn off the lights.  Instead, it
** simply flashes the correct box.
**
**		Furthermore, the button handling scheme is rather more complex:
** instead of simply getting values of 1, 2, 3, or abort, we must now
** coordinate the mouse cursor and the response screen to get values
** from 1 to tokenCount and an abort code.
**
**		The screen is layed out in a 7 x 5 grid of boxes (some of them
** are invisible).  The code gets the box number corresponding to the
** area in which the right mouse button was pressed, then looks it
** up in the list of tokens.  The code returns a non-zero code only
** upon abort or when a valid token-box has been selected.
**
**		In addition, for the monochrome monitor, we must also
** handle our own graphics cursor.  Pretty tricky.  We'll probably
** also end up handling the VGA cursor ourselves, too.  It turns out
** we do not yet know the secret code that lets the mouse driver handle
** the monochrome monitor.  So the mouse is stuck thinking its still in
** our VGA mode (640x200).  Thus, we must do the converting and
** clipping ourselves.
**
** FOR FUTURE:
**
**		We need to make certain that we know the current mouse
** coordinate system.  We should incorporate functions to obtain limits
** from each of the response-button classes.  We also need to be able
** to have these classes employ more restrictive limits as desired.
** LATER.
**
*************************************************************************/

int
ResponseBGITokenDevice::responseFeedback
(
	int stimulus		// number of selected token, 1 to tokenCount
)
{
	int tkey;

	if (!subInterior || !buttons)
		tkey = BOTH_BUTTONS;
	else
	{
		int x = 0, y = 0;						// current mouse coordinates
		int gotbox = 0;							// many things must come true

		subInterior->answerLight();				// light answer light
		subInterior->readyLight(LIGHT_OFF);

		/****************************************************************
		** Turning on cursor here makes it show up quickly at the old
		** location.  So we let cursorMove() below turn it on first,
		** unless the mouse has /not/ moved in the meantime.
		*****************************************************************/

		(void) buttons->readCoordinates(x, y);	// get current coordinates
		if ((x == oldX) && (y == oldY))			// mouse hasn't moved?
			subInterior->cursorOn();			// yes, make cursor visible

		tkey = 0;
		while ((gotbox == 0) && (tkey == 0))
		{
			while (tkey != (int) BUTTON_1)
			{
				tkey = buttons->readCoordinates	// get current coordinates
				(
					x, y						// they're stored here
				);

				if (tkey == BOTH_BUTTONS)		// an abort request
					break;

				if ((x != oldX) || (y != oldY))	// mouse moved since last time
				{
					subInterior->cursorMove(x, y);
					oldX = x;
					oldY = y;
				}
			}

			/************************************************************
			** The normal response is from 1 (BUTTON_1 only) to 7
			** (ALL_BUTTONS).  For this class, it is from 1 to a large
			** number, such as 64.  Therefore, we depend on the caller
			** understanding the situation -- we keep the original set of
			** responses, so that BOTH_BUTTONS==3 has its usual meaning.
			** The other responses are offset upward by 10, and the
			** caller must correct for this fact.  TRICKY CODE.
			*************************************************************/

			if (tkey == BOTH_BUTTONS)			// an abort request
				break;

			tkey = subInterior->getBoxNumber(x, y);
			if (tkey > 0)						// inside the boxes?
			{
				int tokennumber = -1;

				for (int tki = 0; tki < tokenCount; tki++)
				{
					if (tkey == tokenList[tki].box)	// good box number?
					{
						gotbox = 1;					// flag "we got it!"
						tokennumber = tki;			// token with matching box
						break;						// done
					}
				}
				if (gotbox)
				{
					if (tactileFeedback)
					{
						subInterior->cursorOff();			// avoid dirt
						(void) subInterior->feedbackLight	// light it
						(
							tokennumber, tkey
						);
					}
					tkey = tokennumber + TOKEN_KEY_OFFSET;	// offset
				}
				else
					tkey = 0;
			}
		}

		subInterior->cursorOff();					// make sure it's off
		(void) subInterior->answerLight(LIGHT_OFF);

		if ((tkey >= TOKEN_KEY_OFFSET) && selectionFeedback)
		{
			(void) subInterior->feedbackLight		// light feedback light
			(
				(stimulus-1), tkey-TOKEN_KEY_OFFSET	// 0 to tokenCount-1
			);
		}
	}
	return tkey;					// selected box number [re 0] + 10
}

