/************************************************************************
**
** @(#)tvscreen.cpp		10/10/94		Chris Ahlstrom
**
**		Uses TurboVision TTerminal2 class to implement a replacement for
** printf() routines.
**
**		The callers of printf() can instead format their strings first,
** then call screenmsg() to write text to a dumb terminal, where it
** collects so that the user can scroll through it and not miss any
** messages from subroutines.
**
**		I tried to do this myself in tvwindow.cpp, but later saw that
** TV already had its own (and better) version.
**
*************************************************************************/

#define TVSCREEN_cpp

#include <stdio.h>
#include <string.h>
#include <io.h>
#include <alloc.h>
#include <fcntl.h>
#include <strstrea.h>
#include <fstream.h>

#include "tvscreen.h"			// CPP routines for TEST code


/************************************************************************
** TWindowScreen constructor and desctructor
**
**		We could override TWindow::initFrame() for a fancier frame.
**
*************************************************************************/

TWindowScreen::TWindowScreen
(
	const TRect& bounds,		// initial size rectangle for window
	const char *atitle,			// title for the top of the frame
	short anumber,				// bogus number for the window (wmNoNumber)
	ushort bufsize				// buffer size for window
)
 :
	TWindowInit	(TWindowScreen::initFrame),
	TWindow		(bounds, atitle, anumber)
{
	screenInterior = makeInterior(bounds, bufsize);
	if (screenInterior)
		insert(screenInterior);
}


TWindowScreen::~TWindowScreen()
{
    if (screenInterior) 			 // might be deleted by the desktop
	delete screenInterior;
}


/************************************************************************
** TWindowScreen::makeInterior()
*************************************************************************/


TTerminalExtended *
TWindowScreen::makeInterior
(
	TRect bounds,
	ushort bufsize
)
{
	bounds = getExtent();				// size of the window
	bounds.grow(-1, -1);				// interior size matches window
	return new TTerminalExtended
	(
		bounds,
		0,								// no horizontal scrollbar
		standardScrollBar(sbVertical | sbHandleKeyboard),
		bufsize
	);
};


/************************************************************************
** write()
**
**		Writes the contents of the screen to a file.  If the file
** already exists, it is appended to.  If no name is given, TTERM.LOG
** is used.
**
**		The next write() will dump only data added since the previous
** write() call.  The caller can issue a message, if desired.
**
**		Later, we'll figure how to flush and clear.  Right now,
** we just do a killBuffer(), which merely resets the internal
** buffer pointers, effectively emptying the buffer, but showing
** no effect in the window on screen.  So, the caller should be sure
** write a little note telling the user that the screen has indeed
** been written.  See the "Dump Log" command (cmDumpLog) in the face_tv
** sample program.
**
*************************************************************************/

unsigned int
TWindowScreen::write
(
	const char *filename
)
{
	int count = 0;

	if (filename == 0)
		filename = "TTERM.LOG";

	FILE *handle = fopen(filename, "at");

	if (handle)
	{
		count = screenInterior->writeBuffer(handle);

		//
		// if (count > 0)
		//	screenInterior->killBuffer();
		//

		(void) fclose(handle);
	}
	return count;
}

void
TWindowScreen::clear ()
{
	screenInterior->killBuffer();
}


/************************************************************************
** Friend functions of TWindowScreen class
*************************************************************************/

TWindowScreen *
startWindowScreen
(
	TDeskTop *desktop,
	unsigned buffsize,
	int left_x,
	int left_y,
	int right_x,
	int right_y
)
{
	char title[64];							// must match sprintf() string

	if ((coreleft() - 1024) < buffsize)
		buffsize = unsigned(coreleft() - 1024);

	TWindowScreen *crt = new TWindowScreen
	(
		TRect(left_x, left_y, right_x, right_y),
		"Program Log",
		wnNoNumber,
		buffsize
	);
	desktop->insert(crt);

	/*
	** Assign the TTerminal2 interior text device driver to a text "file"
	*/

	otstream os(crt->screenInterior);		// open an output text stream

	// crt->screenOStream = &os;			// save handle to it

	sprintf(title, "TTERMINAL 2 CONSOLE WINDOW, %u bytes\n\n\0", buffsize);
	doScreenMessage(crt, title);			// sort of a sanity check
	crt->screenInterior->scrollTo(0, 0);	// show top of the buffer

	return crt;								// return the handle
}


/************************************************************************
** doScreenMessage()
**
**		This TWindowClass friend simply writes the message to the
** window's stream, then flushes the buffer so the message appears
** right away.
**
**		For some reason, cannot pass the pointer "screenOStream" without
** causing weird problems (see the unuseable code in the USEABLE_CODE
** #if below).
**
*************************************************************************/

void
doScreenMessage
(
	TWindowScreen *handle,
	const char *message
)
{
	otstream os(handle->screenInterior);
	int length = strlen(message);

	if
	(
		(handle != (TWindowScreen *) 0) && (message != NULL) &&
		handle->screenInterior->canInsert(length)
	)
		os << message << flush;
};


#if defined(USEABLE_CODE)								// { USEABLE_CODE

void
doScreenMessage
(
	TWindowScreen *handle,
	char *message
)
{
	if
	(
		(handle != (TWindowScreen *) 0) &&
		(message != NULL) &&
		handle->screenInterior->canInsert(strlen(message))
	)
		*handle->screenOStream << message << flush;
};

#endif													// } USEABLE_CODE



